package com.thaiopensource.pullax;

public interface XmlAttributeItem extends XmlItem {
  /**
   * Gets the index of this attribute within all the attributes
   * of the element.  The first attribute has index 0. The current
   * item must be an attribute.
   *
   * @return the index of the current attribute
   */
  int getAttributeIndex();
  
  /**
   * Indicates whether this attribute was specified in the instance
   * rather than defaulted from the DTD.  If information about whether
   * the attribute was specified is not available, returns
   * <code>true</code>.
   *
   * @return <code>true</code> if this attribute was specified in the instance;
   * <code>false</code> otherwise
   */
  boolean isSpecified();

  /**
   * Determines whether this attribute represents a
   * namespace declaration.
   *
   * @return <code>true</code> if this attribute represents a
   * namespace declaration, <code>false</code> otherwise
   */
  boolean isNamespaceDecl();

  /**
   * Returns null if the attribute is not declared in the DTD.
   */
  XmlAttributeType getAttributeType();

  /**
   * Returns a scanner for the unnormalized value of this attribute.
   *
   * @return an XmlScanner for the unnormalized value of this attribute
   */
  XmlScanner scanUnnormalizedValue();
}
