package com.thaiopensource.pullax;

/**
 * A type-safe enumeration for the XML 1.0 type of an attribute.
 */
public final class XmlAttributeType {
  private final String name;

  private XmlAttributeType(String name) {
    this.name = name;
  }

  public String toString() {
    return name;
  }

  /**
   * The attribute type for attributes declared as <code>CDATA</code>.
   */
  public static final XmlAttributeType CDATA
    = new XmlAttributeType("CDATA");

  /**
   * The attribute type for attributes declared as <code>ID</code>.
   */
  public static final XmlAttributeType ID
    = new XmlAttributeType("ID");

  /**
   * The attribute type for attributes declared as <code>IDREF</code>.
   */
  public static final XmlAttributeType IDREF
    = new XmlAttributeType("IDREF");

  /**
   * The attribute type for attributes declared as <code>IDREFS</code>.
   */
  public static final XmlAttributeType IDREFS
    = new XmlAttributeType("IDREFS");

  /**
   * The attribute type for attributes declared as <code>ENTITY</code>.
   */
  public static final XmlAttributeType ENTITY
    = new XmlAttributeType("ENTITY");

  /**
   * The attribute type for attributes declared as <code>ENTITIES</code>.
   */
  public static final XmlAttributeType ENTITIES
    = new XmlAttributeType("ENTITIES");

  /**
   * The attribute type for attributes declared as <code>NMTOKEN</code>.
   */
  public static final XmlAttributeType NMTOKEN
    = new XmlAttributeType("NMTOKEN");

  /**
   * The attribute type for attributes declared as <code>NMTOKENS</code>.
   */
  public static final XmlAttributeType NMTOKENS
    = new XmlAttributeType("NMTOKENS");

  /**
   * The attribute type for attributes declared as <code>NOTATION</code>.
   */
  public static final XmlAttributeType NOTATION
    = new XmlAttributeType("NOTATION");

  /**
   * The attribute type for attribute whose declared value is a group
   * of NMTOKENs.
   */
  public static final XmlAttributeType ENUMERATION
    = new XmlAttributeType("ENUMERATION");

  public static XmlAttributeType valueOf(String name) {
    if (name.equals("CDATA"))
      return XmlAttributeType.CDATA;
    if (name.equals("ID"))
      return XmlAttributeType.ID;
    if (name.equals("IDREF"))
      return XmlAttributeType.IDREF;
    if (name.equals("IDREFS"))
      return XmlAttributeType.IDREFS;
    if (name.equals("ENTITY"))
      return XmlAttributeType.ENTITY;
    if (name.equals("ENTITIES"))
      return XmlAttributeType.ENTITIES;
    if (name.equals("NMTOKEN"))
      return XmlAttributeType.NMTOKEN;
    if (name.equals("NMTOKENS"))
      return XmlAttributeType.NMTOKENS;
    if (name.equals("NOTATION"))
      return XmlAttributeType.NOTATION;
    if (name.equals("ENUMERATION"))
      return XmlAttributeType.ENUMERATION;
    throw new IllegalArgumentException("illegal attribute type");
  }
}
