package com.thaiopensource.pullax;

public interface XmlContentItem extends XmlItem {
  /**
   * Returns the number of attributes of this item.  If the type of this item
   * is neither <code>START_ELEMENT</code> nor
   * <code>EMPTY_ELEMENT</code>, returns 0.  Default attributes are
   * included.  Attributes serving as namespace declarations are
   * included.
   *
   * @return the number of attributes
   */
  int getAttributeCount();

  /**
   * Returns the attribute with index <code>i</code>.
   * <code>i</code> must be in the range 0 to
   * <code>getAttributeCount() - 1</code>.
   *
   * @param i the index of the attribute to return
   *
   * @throws IndexOutOfBoundsException if <code>i</code> is not in the
   * range 0 to <code>getAttributeCount() - 1</code>
   */
  XmlAttributeItem getAttribute(int i);

  /**
   * Returns the attribute with the specified
   * <code>localName</code> and <code>namespaceUri</code>.  If there
   * is no such attribute, returns null. If the type of this
   * item is not one of <code>START_ELEMENT</code>,
   * <code>EMPTY_ELEMENT</code>, returns null.
   *
   * @param namespaceUri a String giving the namespace URI of the
   * attribute to return; null indicates an absent namespace URI
   * @param localName a String giving the local name of the attribute
   * to return; this must not be null
   */
  XmlAttributeItem getAttribute(String namespaceUri, String localName);

  /**
   * Returns the attribute with the specified
   * <code>localName</code> and no namespace URI. If there
   * is no such attribute, returns null. If the type of this
   * item is not one of <code>START_ELEMENT</code>,
   * <code>EMPTY_ELEMENT</code>, returns null.
   *
   * @param localName a String giving the local name of the attribute
   * @return an XmlAttributeItem for the attribute with the specified
   * local name
   *
   * @see #getAttribute(String,String)
   */
  XmlAttributeItem getAttribute(String localName);
}
