package com.thaiopensource.pullax;

import java.util.Map;

public interface XmlContext {
  /**
   * Returns the base URI of this item.  Returns null if the base URI
   * is unknown.
   *
   * @return a String containing the base URI of the this item
   */
  String getBaseUri();

  /**
   * Returns the name of the character encoding scheme used
   * to encode the external entity containing this item.
   * Returns null if the character encoding scheme is unknown.
   *
   * @return a String containing the name of the character encoding scheme
   */
  String getCharacterEncodingScheme();

  /**
   * Returns the DTD of the document or null if the DTD is not yet
   * known. The DTD is known as soon as the document type declaration
   * has been read or the start-tag of the document element is seen.
   *
   * @return an XmlDtd object representing the DTD of the document
   */
  XmlDtd getDtd();

  /**
   * Returns the version in the XML declaration or text declaration of
   * the external entity containing this object, or null if there
   * was no XML declaration or text declaration, or if the text declaration
   * did not specify a version.
   */
  String getVersion();

  /**
   * Returns an unmodifiable map representing the in-scope namespaces.
   * The map maps namespace prefixes to namespace URIs. The value for
   * a null key gives the default namespace. The map will always include
   * an entry for the prefix <code>xml</code>.
   */
  Map getInScopeNamespaces();

  /**
   * A type-safe enumeration for the possible values of the
   * <code>standalone</code> pseudo-attribute in the XML declaration.
   */
  public static final class Standalone {
    private final String value;

    private Standalone(String value) {
      this.value = value;
    }

    public String toString() {
      return value;
    }

    public static final Standalone YES = new Standalone("yes");
    public static final Standalone NO = new Standalone("no");
  }

  Standalone getStandalone();

  /**
   * Returns the current applicable value of the <code>xml:lang</code>
   * attribute, or null if there is no appicable value.
   */
  String getXmlLang();
  
  /**
   * A type-safe enumeration for the valid values of an <code>xml:space</code>
   * attribute.
   */
  public static final class XmlSpace {
    private final String value;

    private XmlSpace(String value) {
      this.value = value;
    }

    public String toString() {
      return value;
    }

    public static final XmlSpace DEFAULT = new XmlSpace("default");
    public static final XmlSpace PRESERVE = new XmlSpace("preserve");
  }

  /**
   * Returns the current applicable value of the <code>xml:space</code>
   * attribute, or null if there is no applicable value or the
   * applicable value is invalid.
   */
  XmlSpace getXmlSpace();
}
