package com.thaiopensource.pullax;

import java.util.ResourceBundle;
import java.util.Locale;
import java.text.MessageFormat;

public class XmlReportFormat {
  private ResourceBundle bundle;

  private static final String BUNDLE_NAME
    = "com.thaiopensource.pullax.resources.Messages";
    
  protected XmlReportFormat(ResourceBundle bundle) {
    this.bundle = bundle;
  }

  public String format(XmlReport report) {
    return format(report.getSeverity().toString(),
		  new Object[] { formatLocation(report.getLocation()),
				 report.getMessage() });
  }
  
  private String format(String key, Object[] args) {
    return MessageFormat.format(getString(key), args);
  }

  private String formatLocation(XmlLocation loc) {
    String uri = loc.getUri();
    int n = loc.getLineNumber();
    Integer lineNumber = n >= 0 ? new Integer(n) : null;
    n = loc.getColumnNumber();
    Integer columnNumber = n >= 0 ? new Integer(n) : null;
    if (uri != null) {
      if (lineNumber != null) {
	if (columnNumber != null)
	  return format("location_uri_line_column",
			new Object[] { uri, lineNumber, columnNumber });
	else
	  return format("location_uri_line",
			new Object[] { uri, lineNumber });
      }
      else
	return format("location_uri",
		      new Object[] { uri });
    }
    else if (lineNumber != null) {
      if (columnNumber != null)
	return format("location_line_column",
		      new Object[] { lineNumber, columnNumber });
      else
	return format("location_line",
		      new Object[] { lineNumber });
    }
    else
      return getString("location_empty");
  }

  private String getString(String key) {
    return bundle.getString(key);
  }
  
  static public XmlReportFormat getInstance() {
    return getInstance(Locale.getDefault());
  }

  static public XmlReportFormat getInstance(Locale locale) {
    return new XmlReportFormat(ResourceBundle.getBundle(BUNDLE_NAME, locale));
  }
}
