package com.thaiopensource.pullax;

import java.io.IOException;
import java.util.Locale;

public interface XmlScannerFactory {
  XmlScanner createScanner(XmlInputSource input) throws IOException;
  /**
   * Adds the specified features to the factory
   * and removes all other features.
   * Also adds the features implied by the specified features.
   * @throws XmlNotSupportedException if not all required features are
   * included in features
   */
  void setFeatures(XmlFeature[] features)
    throws XmlNotSupportedException;
  /**
   * Sometimes a particular value for one feature must imply a particular
   * value for another feature.  In this case, adding one feature
   * to that value should automatically set any features it implies.
   */
  void addFeature(XmlFeature feature) throws XmlNotSupportedException;
  boolean hasFeature(XmlFeature feature);
  void setEntityResolver(XmlEntityResolver resolver);
  XmlEntityResolver getEntityResolver();
  void setReporter(XmlReporter reporter);
  XmlReporter getReporter();
  XmlScannerFactory copy();

  void setLocale(Locale locale);
  Locale getLocale();
}
