package com.thaiopensource.pullax.util;

import java.util.Map;
import java.util.Iterator;
import java.io.IOException;

import com.thaiopensource.pullax.XmlFeature;
import com.thaiopensource.pullax.XmlScanner;
import com.thaiopensource.pullax.XmlScannerFactory;
import com.thaiopensource.pullax.XmlScannerFactoryFinder;
import com.thaiopensource.pullax.XmlContentItem;
import com.thaiopensource.pullax.XmlAttributeItem;
import com.thaiopensource.pullax.XmlItem;
import com.thaiopensource.pullax.XmlDtd;
import com.thaiopensource.pullax.XmlContext;
import com.thaiopensource.pullax.XmlLocation;
import com.thaiopensource.pullax.XmlFileInputSource;
import com.thaiopensource.pullax.XmlNotSupportedException;
import com.thaiopensource.pullax.XmlReportException;
import com.thaiopensource.pullax.XmlReporter;
import com.thaiopensource.pullax.XmlReport;
import com.thaiopensource.pullax.XmlReportFormat;

import org.apache.xerces.xni.XNIException;

public class Dump {

  private Dump() { }

  private static XmlFeature features[] = {
    XmlFeature.SHOW_DOCUMENT_ENTITY,
    XmlFeature.SHOW_EXTERNAL_ENTITY,
    XmlFeature.SHOW_DOCUMENT_TYPE,
    XmlFeature.ALLOW_SPLIT_TEXT,
    XmlFeature.SHOW_CDATA_SECTION,
    XmlFeature.SHOW_CHARACTER_REFERENCE,
    XmlFeature.SHOW_BUILTIN_ENTITY_REFERENCE
  };

  public static void main(String[] args) throws IOException, XmlNotSupportedException {
    XmlScannerFactory factory = XmlScannerFactoryFinder.findScannerFactory(features);
    factory.setReporter(new XmlReporter() {
	public void report(XmlReport report) {
	  System.err.println(XmlReportFormat.getInstance().format(report));
	}
      });
    XmlContext context = null;
    for (XmlScanner scanner = factory.createScanner(new XmlFileInputSource(args[0]));
	 scanner.hasMoreItems();) {
      XmlContentItem item = scanner.nextItem();
      if (item.getContext() != context) {
	context = item.getContext();
	printContext(context);
      }
      printItem(item);
    }
  }
  
  private static void printItem(XmlContentItem item) throws IOException {
    printLoc(item.getLocation());
    switch (item.getItemType()) {
    case XmlItem.START_ELEMENT:
      System.out.print("<start-element");
      printName(item);
      System.out.println(">");
      printAtts(item);
      System.out.println("</start-element>");
      break;
    case XmlItem.END_ELEMENT:
      System.out.print("<end-element");
      printName(item);
      System.out.println("/>");
      break;
    case XmlItem.START_ENTITY:
      System.out.print("<start-entity");
      if (item.getName() != null)
	printName(item);
      System.out.println("/>");
      break;
    case XmlItem.CHARACTER_REFERENCE:
      System.out.print("<character-reference");
      printName(item);
      System.out.print(">");
      printText(item.getValue(), false);
      System.out.println("</character-reference>");
      break;
    case XmlItem.BUILTIN_ENTITY_REFERENCE:
      System.out.print("<builtin-entity-reference");
      printName(item);
      System.out.print(">");
      printText(item.getValue(), false);
      System.out.println("</builtin-entity-reference>");
      break;
    case XmlItem.END_ENTITY:
      System.out.print("<end-entity");
      if (item.getName() != null)
	printName(item);
      System.out.println("/>");
      break;
    case XmlItem.EMPTY_ELEMENT:
      System.out.print("<empty-element");
      printName(item);
      System.out.println(">");
      printAtts(item);
      System.out.println("</empty-element>");
      break;
    case XmlItem.TEXT:
      System.out.print("<text");
      System.out.print('>');
      printText(item.getValue(), false);
      System.out.println("</text>");
      break;
    case XmlItem.CDATA_SECTION:
      System.out.print("<cdata-section");
      System.out.print('>');
      printText(item.getValue(), false);
      System.out.println("</cdata-section>");
      break;
    case XmlItem.COMMENT:
      System.out.print("<comment");
      System.out.print('>');
      System.out.print(item.getValue());
      System.out.println("</comment>");
      break;
    case XmlItem.PROCESSING_INSTRUCTION:
      System.out.print("<processing-instruction");
      printName(item);
      System.out.print(">");
      printText(item.getValue(), false);
      System.out.println("</processing-instruction>");
      break;
    case XmlItem.DOCUMENT_TYPE:
      System.out.print("<document-type");
      printName(item);
      System.out.println(">");
      printDtd(item.getContext().getDtd());
      System.out.println("</document-type>");
      break;
    default:
      System.err.println("unrecognized item type " + item.getItemType());
      break;
    }
  }

  private static void printAtts(XmlContentItem item) throws IOException {
    for (int i = 0, n = item.getAttributeCount(); i < n; i++) {
      XmlAttributeItem att = item.getAttribute(i);
      System.out.print(att.isNamespaceDecl()
		       ? "<namespace-decl"
		       : "<attribute");
      printName(att);
      System.out.print(" value=\"");
      printText(att.getValue(), true);
      System.out.println("\">");
      for (XmlScanner scanner = att.scanUnnormalizedValue();
	   scanner.hasMoreItems();)
	printItem(scanner.nextItem());
      System.out.println(att.isNamespaceDecl()
			 ? "</namespace-decl>"
			 : "</attribute>");
    }
  }

  private static void printName(XmlItem item) {
    System.out.print(" local-name=\"" + item.getLocalName() + "\"");
    String uri = item.getNamespaceUri();
    if (uri != null)
      System.out.print(" namespace-uri=\"" + item.getNamespaceUri() + "\"");
    String prefix = item.getPrefix();
    if (prefix != null)
      System.out.print(" prefix=\"" + item.getPrefix() + "\"");
  }

  private static void printContext(XmlContext context) {
    System.out.print("<context base-uri=\"" + context.getBaseUri() + "\"");
    if (context.getStandalone() != null)
      System.out.print(" standalone=\"" + context.getStandalone().toString() + "\"");
    if (context.getXmlSpace() != null)
      System.out.print(" xml-space=\"" + context.getXmlSpace().toString() + "\"");
    if (context.getXmlLang() != null)
      System.out.print(" xml-lang=\"" + context.getXmlLang() + "\"");
    System.out.println(">");
    for (Iterator iter = context.getInScopeNamespaces().entrySet().iterator();
	 iter.hasNext();) {
      Map.Entry entry = (Map.Entry)iter.next();
      System.out.print("<namespace uri=\"" + (String)entry.getValue() + "\"");
      if (entry.getKey() != null)
	System.out.print(" prefix=\"" + (String)entry.getKey() + "\"");
      System.out.println("/>");
    }
    System.out.println("</context>");
  }

  private static void printLoc(XmlLocation loc) {
    System.out.println("<location uri=\"" + loc.getUri() +
		       "\" line=\"" + loc.getLineNumber() +
		       "\" column=\"" + loc.getColumnNumber() +
		       "\"/>");
  }

  private static void printText(String s, boolean inAttribute) {
    for (int i = 0, n = s.length(); i < n; i++) {
      char c = s.charAt(i);
      switch (c) {
      case '&':
	System.out.print("&amp;");
	break;
      case '<':
	System.out.print("&lt;");
	break;
      case '>':
	System.out.print("&gt;");
	break;
      case '\n':
	System.out.print("&#xA;");
	break;
      case '\t':
	System.out.print("&#x9;");
	break;
      case '\r':
	System.out.print("&#xD;");
	break;
      case '"':
	if (inAttribute) {
	  System.out.print("&quot;");
	  break;
	}
	// fall through
      default:
	System.out.print(c);
	break;
      }
    }
  }

  private static void printDtd(XmlDtd dtd) throws IOException {
    for (XmlScanner scanner = dtd.scanProcessingInstructions();
	 scanner.hasMoreItems();)
      printItem(scanner.nextItem());
    String s = dtd.getInternalSubset();
    if (s != null) {
      System.out.print("<internal-subset><![CDATA[");
      System.out.print(s);
      System.out.println("]]></internal-subset>");
    }
  }
}
