package com.thaiopensource.pullax.xni;

import com.thaiopensource.pullax.XmlContext;
import com.thaiopensource.pullax.XmlDtd;
import com.thaiopensource.pullax.XmlNamespaces;

import java.util.Map;
import java.util.HashMap;
import java.util.Collections;

class Context implements XmlContext {
  private boolean frozen = false;
  private String sourceUri;
  private String baseUri;
  private String characterEncodingScheme;
  private XmlDtd dtd;
  private String version;
  private Map inScopeNamespaces;
  private Standalone standalone;
  private XmlSpace xmlSpace;
  private String xmlLang;

  Context() {
    inScopeNamespaces = new HashMap();
    inScopeNamespaces.put("xml", XmlNamespaces.XML);
  }

  Context(Context context) {
    sourceUri = context.sourceUri;
    baseUri = context.baseUri;
    characterEncodingScheme = context.characterEncodingScheme;
    dtd = context.dtd;
    version = context.version;
    inScopeNamespaces = context.inScopeNamespaces;
    standalone = context.standalone;
    xmlSpace = context.xmlSpace;
    xmlLang = context.xmlLang;
  }

  void freeze() {
    frozen = true;
    if (inScopeNamespaces.getClass() == HashMap.class)
      inScopeNamespaces = Collections.unmodifiableMap(inScopeNamespaces);
  }

  Context setSourceUri(String s) {
    if (!same(s, sourceUri)) {
      if (frozen)
        return new Context(this).setSourceUri(s);
      sourceUri = s;
    }
    return this;
  }

  String getSourceUri() {
    return sourceUri;
  }

  Context setBaseUri(String s) {
    if (!same(s, baseUri)) {
      if (frozen)
        return new Context(this).setBaseUri(s);
      baseUri = s;
    }
    return this;
  }

  public String getBaseUri() {
    return baseUri;
  }

  Context setCharacterEncodingScheme(String s) {
    if (!same(s, characterEncodingScheme)) {
      if (frozen)
        return new Context(this).setSourceUri(s);
      characterEncodingScheme = s;
    }
    return this;
  }


  public String getCharacterEncodingScheme() {
    return characterEncodingScheme;
  }

  Context setDtd(XmlDtd d) {
    if (d != dtd) {
      if (frozen)
        return new Context(this).setDtd(d);
      dtd = d;
    }
    return this;
  }

  public XmlDtd getDtd() {
    return dtd;
  }

  Context setVersion(String s) {
    if (!same(s, version)) {
      if (frozen)
        return new Context(this).setVersion(s);
      version = s;
    }
    return this;
  }

  public String getVersion() {
    return version;
  }
  
  Context setStandalone(Standalone s) {
    if (s != standalone) {
      if (frozen)
	return new Context(this).setStandalone(s);
      standalone = s;
    }
    return this;
  }

  public Standalone getStandalone() {
    return standalone;
  }

  Context setXmlSpace(XmlSpace s) {
    if (s != xmlSpace) {
      if (frozen)
	return new Context(this).setXmlSpace(s);
      xmlSpace = s;
    }
    return this;
  }

  public XmlSpace getXmlSpace() {
    return xmlSpace;
  }

  Context setXmlLang(String s) {
    if (!same(s, xmlLang)) {
      if (frozen)
	return new Context(this).setXmlLang(s);
      xmlLang = s;
    }
    return this;
  }

  public String getXmlLang() {
    return xmlLang;
  }

  Context setNamespace(String prefix, String uri) {
    if (!same(uri, (String)inScopeNamespaces.get(prefix))) {
      if (frozen)
	return new Context(this).setNamespace(prefix, uri);
      if (inScopeNamespaces.getClass() != HashMap.class)
	inScopeNamespaces = new HashMap(inScopeNamespaces);
      if (uri == null)
	inScopeNamespaces.remove(prefix);
      else
	inScopeNamespaces.put(prefix, uri);
    }
    return this;
  }

  public Map getInScopeNamespaces() {
    return inScopeNamespaces;
  }

  static private boolean same(String s1, String s2) {
    if (s1 == null)
      return s2 == null;
    else
      return s1.equals(s2);
  }
}
